#include <signal.h>
#include <unistd.h>
#include <stdbool.h>
#include <stdlib.h>
#include <stdio.h>

static void catcher(int signo) {
  struct sigaction sig;

  switch (signo) {
    case SIGHUP:
      printf("Hang-up signal (SIGHUP) caught.\n");
      break;
    case SIGINT:
      printf("Interactive attention signal (SIGINT) caught.\n");
      break;
    case SIGTERM:
      printf("Termination signal (SIGTERM) caught.\n");
      exit(0);
      break;
    case SIGALRM:
      printf("Alarm signal (SIGALRM) caught.\n");
      break;
    case SIGTSTP:
      printf("Stop signal (SIGTSTP) caught.\n");
      sig.sa_handler = SIG_DFL;
      sig.sa_flags = SA_RESTART;
      sigemptyset (&sig.sa_mask);
      if (sigaction(signo, &sig, NULL) < 0) {
        fputs("An error occurred while installing signal handler.\n", stderr);
        exit(1);
      }
      raise(signo);
      break;
    case SIGCONT:
      printf("Continue signal (SIGCONT) caught.\n");
      sig.sa_handler = catcher;
      sig.sa_flags = SA_RESTART;
      sigemptyset (&sig.sa_mask);
      if (sigaction(SIGTSTP, &sig, NULL) < 0) {
        fputs("An error occurred while installing signal handler.\n", stderr);
        exit(1);
      }
      break;
    case SIGUSR1:
      printf("User defined signal 1 (SIGUSR1) caught.\n");
      break;
    case SIGUSR2:
      printf("User defined signal 2 (SIGUSR2) caught.\n");
      printf("Begin SIGUSR2 handling...\n");
      raise(SIGINT);
      printf("End SIGUSR2 handling...\n");
      break;
    default:
      printf("Signal %i caught.\n", signo);
  }
}
 
int main(void) {
  struct sigaction sig;
  int sigs_to_handle[] = { SIGINT, SIGTERM, SIGTSTP, SIGCONT, SIGUSR2 };
  int i;
  sig.sa_handler = catcher;
  sig.sa_flags = SA_RESTART;
  sigemptyset (&sig.sa_mask);
  sigaddset (&sig.sa_mask, SIGTERM);

  for(i=0; i<(sizeof(sigs_to_handle)/sizeof(int)); i++) {
    if (sigaction(sigs_to_handle[i], &sig, NULL) < 0) {
      fputs("An error occurred while installing signal handler.\n", stderr);
      return 1;
    }
  }

  printf("Raising the interactive attention signal (SIGINT).\n");
  if (raise(SIGINT) != 0) {
    fputs("Error raising the signal.\n", stderr);
    return 1;
  }
  printf("Raising the User defined signal 2 (SIGUSR2).\n");
  if (raise(SIGUSR2) != 0) {
    fputs("Error raising the signal.\n", stderr);
    return 1;
  }
  printf("Looping infinite...\n");
  while(true)
    sleep(60);

  return 0;
}

